# -*- coding: utf-8 -*-
# License: Apache License 2.0
"""Classes related to the different supported model types.

    :author: Patrick L. Lange (plange@ets.org)
    :author: Rutuja Ubale (rutuja@ucla.edu)
    :organization: ETS

"""
from abc import ABCMeta, abstractmethod
from skll import Learner
from features import bow


class Predictions():
    """Class to hold the predictions.

    This should be extended to convert the predictions into desired formats.

    """
    def __init__(self, predictions):
        self.predictions = predictions


class Model():
    """Abstract SLU model.

    Initializes using the __init__ method and generates a Prediction object
    using the predictions method.

    """
    __metaclass__ = ABCMeta

    def __init__(self, name, learner_file, classes):
        self.name = name
        self.learner = Learner.from_file(learner_file)
        self.classes = sorted(classes)

    @abstractmethod
    def predict(self, words_list):
        pass


class BowModel(Model):
    """Bag of word SLU model."""

    def __init__(self, name, classes, learner_file, vocab_file):
        """Creates a bag of word SLU model.

        Args:
            name (string): Model name
            classes (list of strings): class names
            learner_file (path): skll model file to use
            vocab_file (path): vocabulary file to use

        Returns:
            BowModel

        Notes:
            The classes will be sorted alphabetically therefore this should
            also be done during training.

        """
        with open(vocab_file, 'r') as fh:
            self.vocab = [line.rstrip() for line in fh]
        super(BowModel, self).__init__(name, learner_file, classes)

    def predict(self, words_list):
        """Creates predictions using the model.

        Args:
            words_list(list of word arrays): Each sentence should be a
                word array.

        Returns:
            Prediction

        """
        probs = self.learner.predict(bow(words_list, self.vocab))
        predictions = Predictions(zip([" ".join(words) for words in words_list],
                                  [zip(self.classes, p) for p in probs]))
        return predictions
