# -*- coding: utf-8 -*-
# License: Apache License 2.0
"""API implementation.

    :author: Patrick L. Lange (plange@ets.org)
    :author: Rutuja Ubale (rutuja@ucla.edu)
    :organization: ETS

"""
from halef_setu_api import app, slumodels
from flask import abort, jsonify, request
import re

@app.route('/<string:item>/<string:state>')
def get_slu(item, state):
    """API for receiving the SLU predictions.

    Args:
        item (string): application identifier
        state (string): dialog state identifier

    Result:
        Returns JSON containing the original text input, the normalized text,
        the item and state name as well as the classes with their probabilities.
        The classes are sorted from the class with the highest to the class with
        the lowest probability.

    Note:
        The API expects the text to be classified as GET parameter with
        parameter name "text". This means the text needs to be URL encoded.

    Example:
        The following example calls the API using the model identified by
        <item-name> and <dialog-state-name> and returns the prediction for the
        URL encoded text "This is an example."

        http://<domain>:<port>/<item-name>/<dialog-state-name>/
            text?This%20is%20an%20example.

    """
    # Get query string or return Bad Request error
    if 'text' in request.args:
        text = request.args.get('text')
    else:
        abort(400)

    # Normalize the input
    normalize = re.sub('[^a-z0-9,_ \n]', '', text.lower().replace("'", " "))
    normalize = normalize.split(',')[0]
    words_list = [normalize.split()]

    # Check if requested model is loaded, else Not Found error
    if (item, state) in slumodels:
        # Compute predictions
        results = slumodels[(item, state)].predict(words_list).predictions

        # Format response
        json_result = {}
        json_result['item'] = item
        json_result['state'] = state
        json_result['original'] = text
        json_result['processed'] = results[0][0]
        classes = {}
        for i, pair in enumerate(sorted(results[0][1],
                                        key=lambda x: x[1],
                                        reverse=True)):
            classes[i] = {'name': pair[0], 'prob': pair[1]}
        json_result['classes'] = classes
        return jsonify(json_result)
    else:
        abort(404)
